@@ -25,3 +25,5 @@ MIDDLEWARE_CLASSES = (
 
 ROOT_URLCONF = 'urls'
 CRISPY_TEMPLATE_PACK = 'bootstrap'
+CRISPY_CLASS_CONVERTERS = {"textinput": "textinput textInput inputtext"}
+SECRET_KEY = 'secretkey'
