@@ -6,7 +6,7 @@ from django import forms
 from django.conf import settings
 from django.core.urlresolvers import reverse
 from django.forms.forms import BoundField
-from django.forms.models import formset_factory
+from django.forms.models import formset_factory, modelformset_factory
 from django.template import Context, TemplateSyntaxError
 from django.template.loader import get_template_from_string
 from django.middleware.csrf import _get_new_csrf_key
@@ -30,7 +30,7 @@ from crispy_forms.templatetags.crispy_forms_tags import CrispyFormNode
 
 from .forms import (
     TestForm, TestForm2, TestForm3, ExampleForm, CheckboxesTestForm,
-    FormWithMeta, TestForm4
+    FormWithMeta, TestForm4, CrispyTestModel
 )
 
 
@@ -524,6 +524,43 @@ class TestFormHelpers(TestCase):
 
         self.assertFalse("<input type='hidden' name='csrfmiddlewaretoken'" in html)
 
+    def test_render_hidden_fields(self):
+        test_form = TestForm()
+        test_form.helper = FormHelper()
+        test_form.helper.layout = Layout(
+            'email'
+        )
+        test_form.helper.render_hidden_fields = True
+
+        html = render_crispy_form(test_form)
+        self.assertEqual(html.count('<input'), 1)
+
+        # Now hide a couple of fields
+        for field in ('password1', 'password2'):
+            test_form.fields[field].widget = forms.HiddenInput()
+
+        html = render_crispy_form(test_form)
+        self.assertEqual(html.count('<input'), 3)
+        self.assertEqual(html.count('hidden'), 2)
+
+        if django.get_version() < '1.5':
+            self.assertEqual(html.count('type="hidden" name="password1"'), 1)
+            self.assertEqual(html.count('type="hidden" name="password2"'), 1)
+        else:
+            self.assertEqual(html.count('name="password1" type="hidden"'), 1)
+            self.assertEqual(html.count('name="password2" type="hidden"'), 1)
+
+    def test_render_required_fields(self):
+        test_form = TestForm()
+        test_form.helper = FormHelper()
+        test_form.helper.layout = Layout(
+            'email'
+        )
+        test_form.helper.render_required_fields = True
+
+        html = render_crispy_form(test_form)
+        self.assertEqual(html.count('<input'), 7)
+
 
 class TestFormLayout(TestCase):
     urls = 'crispy_forms.tests.urls'
@@ -861,45 +898,56 @@ class TestFormLayout(TestCase):
         self.assertFalse('email' in html)
 
     def test_formset_layout(self):
-        template = get_template_from_string(u"""
-            {% load crispy_forms_tags %}
-            {% crispy testFormSet formset_helper %}
-        """)
-
-        form_helper = FormHelper()
-        form_helper.form_id = 'thisFormsetRocks'
-        form_helper.form_class = 'formsets-that-rock'
-        form_helper.form_method = 'POST'
-        form_helper.form_action = 'simpleAction'
-        form_helper.add_layout(
-            Layout(
-                Fieldset("Item {{ forloop.counter }}",
-                    'is_company',
-                    'email',
-                ),
-                HTML("{% if forloop.first %}Note for first form only{% endif %}"),
-                Row('password1', 'password2'),
-                Fieldset("",
-                    'first_name',
-                    'last_name'
-                )
+        TestFormSet = formset_factory(TestForm, extra=3)
+        formset = TestFormSet()
+        helper = FormHelper()
+        helper.form_id = 'thisFormsetRocks'
+        helper.form_class = 'formsets-that-rock'
+        helper.form_method = 'POST'
+        helper.form_action = 'simpleAction'
+        helper.layout = Layout(
+            Fieldset("Item {{ forloop.counter }}",
+                'is_company',
+                'email',
+            ),
+            HTML("{% if forloop.first %}Note for first form only{% endif %}"),
+            Row('password1', 'password2'),
+            Fieldset("",
+                'first_name',
+                'last_name'
             )
         )
 
-        TestFormSet = formset_factory(TestForm, extra = 3)
-        testFormSet = TestFormSet()
+        html = render_crispy_form(
+            form=formset, helper=helper, context={'csrf_token': _get_new_csrf_key()}
+        )
 
-        c = Context({
-            'testFormSet': testFormSet,
-            'formset_helper': form_helper,
-            'csrf_token': _get_new_csrf_key()
-        })
-        html = template.render(c)
+        # Check formset fields
+        if django.get_version() < '1.5':
+            self.assertEqual(html.count(
+                'type="hidden" name="form-TOTAL_FORMS" value="3" id="id_form-TOTAL_FORMS"'
+            ), 1)
+            self.assertEqual(html.count(
+                'type="hidden" name="form-INITIAL_FORMS" value="0" id="id_form-INITIAL_FORMS"'
+            ), 1)
+            self.assertEqual(html.count(
+                'type="hidden" name="form-MAX_NUM_FORMS" id="id_form-MAX_NUM_FORMS"'
+            ), 1)
+        else:
+            self.assertEqual(html.count(
+                'id="id_form-TOTAL_FORMS" name="form-TOTAL_FORMS" type="hidden" value="3"'
+            ), 1)
+            self.assertEqual(html.count(
+                'id="id_form-INITIAL_FORMS" name="form-INITIAL_FORMS" type="hidden" value="0"'
+            ), 1)
+            self.assertEqual(html.count(
+                'id="id_form-MAX_NUM_FORMS" name="form-MAX_NUM_FORMS" type="hidden" value="1000"'
+            ), 1)
+        self.assertEqual(html.count("hidden"), 4)
 
-        # Check form parameters
+        # Check form structure
         self.assertEqual(html.count('<form'), 1)
         self.assertEqual(html.count("<input type='hidden' name='csrfmiddlewaretoken'"), 1)
-
         self.assertTrue('formsets-that-rock' in html)
         self.assertTrue('method="post"' in html)
         self.assertTrue('id="thisFormsetRocks"' in html)
@@ -910,12 +958,52 @@ class TestFormLayout(TestCase):
         self.assertTrue('Item 2' in html)
         self.assertTrue('Item 3' in html)
         self.assertEqual(html.count('Note for first form only'), 1)
-
         if settings.CRISPY_TEMPLATE_PACK == 'uni_form':
             self.assertEqual(html.count('formRow'), 3)
         else:
             self.assertEqual(html.count('row'), 3)
 
+    def test_modelformset_layout(self):
+        CrispyModelFormSet = modelformset_factory(CrispyTestModel, form=TestForm4, extra=3)
+        formset = CrispyModelFormSet(queryset=CrispyTestModel.objects.none())
+        helper = FormHelper()
+        helper.layout = Layout(
+            'email'
+        )
+
+        html = render_crispy_form(form=formset, helper=helper)
+        self.assertEqual(html.count("id_form-0-id"), 1)
+        self.assertEqual(html.count("id_form-1-id"), 1)
+        self.assertEqual(html.count("id_form-2-id"), 1)
+
+        if django.get_version() < '1.5':
+            self.assertEqual(html.count(
+                'type="hidden" name="form-TOTAL_FORMS" value="3" id="id_form-TOTAL_FORMS"'
+            ), 1)
+            self.assertEqual(html.count(
+                'type="hidden" name="form-INITIAL_FORMS" value="0" id="id_form-INITIAL_FORMS"'
+            ), 1)
+            self.assertEqual(html.count(
+                'type="hidden" name="form-MAX_NUM_FORMS" id="id_form-MAX_NUM_FORMS"'
+            ), 1)
+
+        else:
+            self.assertEqual(html.count(
+                'id="id_form-TOTAL_FORMS" name="form-TOTAL_FORMS" type="hidden" value="3"'
+            ), 1)
+            self.assertEqual(html.count(
+                'id="id_form-INITIAL_FORMS" name="form-INITIAL_FORMS" type="hidden" value="0"'
+            ), 1)
+            self.assertEqual(html.count(
+                'id="id_form-MAX_NUM_FORMS" name="form-MAX_NUM_FORMS" type="hidden" value="1000"'
+            ), 1)
+
+        self.assertEqual(html.count('name="form-0-email"'), 1)
+        self.assertEqual(html.count('name="form-1-email"'), 1)
+        self.assertEqual(html.count('name="form-2-email"'), 1)
+        self.assertEqual(html.count('name="form-3-email"'), 0)
+        self.assertEqual(html.count('password'), 0)
+
     def test_multiwidget_field(self):
         template = get_template_from_string(u"""
             {% load crispy_forms_tags %}
@@ -1003,7 +1091,6 @@ class TestLayoutObjects(TestCase):
             {% crispy test_form %}
         """)
 
-
         test_form = TestForm()
         test_form.helper = FormHelper()
         test_form.helper.layout = Layout(
@@ -1028,11 +1115,6 @@ class TestLayoutObjects(TestCase):
             self.assertEqual(html.count('class="control-group testing"'), 1)
 
     def test_appended_prepended_text(self):
-        template = get_template_from_string(u"""
-            {% load crispy_forms_tags %}
-            {% crispy test_form %}
-        """)
-
         test_form = TestForm()
         test_form.helper = FormHelper()
         test_form.helper.layout = Layout(
@@ -1040,11 +1122,7 @@ class TestLayoutObjects(TestCase):
             AppendedText('password1', '#'),
             PrependedText('password2', '$'),
         )
-
-        c = Context({
-            'test_form': test_form,
-        })
-        html = template.render(c)
+        html = render_crispy_form(test_form)
 
         # Check form parameters
         self.assertEqual(html.count('<span class="add-on">@</span>'), 1)
